#!/usr/bin/python

# Copyright

import os.path

import cookbook
from cookbook import Cookbook
from cookbook.mom import MomParser
from cookbook.cookbook import test as test_cookbook

try:
    import cherrypy
    from cookbook.server import Server
    from cookbook.server import test as test_server
except ImportError, e:
    cherrypy = e
    def test_server():
        pass


if __name__ == '__main__':
    import optparse
    import sys

    # HACK! to ensure we *always* get utf-8 output
    p = optparse.OptionParser()
    p.add_option('-t', '--test', dest='test', default=False,
                 action='store_true', help='run internal tests and exit')
    p.add_option('-m', '--mom', dest='mom', metavar='PATH',
                 help="load Mom's cookbook")
    p.add_option('-s', '--serve', dest='serve', default=False,
                 action='store_true', help='serve cookbook')
    p.add_option('-a', '--address', dest='address', default='127.0.0.1',
                 metavar='ADDR', help='address that the server will bind to')
    p.add_option('-p', '--port', dest='port', default='8080',
                 metavar='PORT', help='port that the server will listen on')

    options,arguments = p.parse_args()

    if options.test == True:
        test_cookbook()
        test_server()
        sys.exit(0)

    if isinstance(cherrypy, ImportError):
        raise cherrypy

    sys.stderr.write('Loading cookbook\n')
    if options.mom == None:
        c = Cookbook()
        c.load()
    else:
        p = MomParser()
        c = p.parse(options.mom)

    if options.serve == False:
        sys.stderr.write('Saving cookbook\n')
        c.save('new-recipe')
    else:
        #reload(sys)
        #sys.setdefaultencoding('utf-8')
        sys.stderr.write('Serving cookbook\n')
        module_dir = os.path.dirname(os.path.abspath(cookbook.__file__))
        static_dir = os.path.join(module_dir, 'static')
        template_dir = os.path.join(module_dir, 'template')
        config = os.path.join(module_dir, 'config')
        s = Server(c, template_dir)
        cherrypy.config.update({ # http://www.cherrypy.org/wiki/ConfigAPI
                'server.socket_host': options.address,
                'server.socket_port': int(options.port),
                'tools.encode.on': True,
                'tools.encode.encoding': 'utf8',
                'tools.staticdir.root': static_dir
                })
        app_config = { '/static': { 'tools.staticdir.on': True,
                                    'tools.staticdir.dir': '', } }
        cherrypy.quickstart(root=s, config=app_config)
        sys.stderr.write('Saving cookbook\n')
        s.cleanup()
